/* anatabacus.js: Dialog for EPI distortion correction plugin 'anatabacus' 
Content: This EPI distortion correction bundle consists of anatabacus.js, anatabacus.ui and anatabacus.dylib/dll/so. 
Installation: place all in /Documents/BVExtensions/Plugins_32/64/ folder.
Functionality: Undistort functional EPI data.
Usage: In the BrainVoyager menu, select 'Plugins' > 'anatabacus...'.

History: 
 
 >> remove datatoselect file
13-09-18: improved closeAllProjects() 
30-01-18:
- changed place of 'Convert to radians' checkbox to first tab, and made independent of unwrapping, placed collection of parameter in collectAcquisitionParameters().
15-01-18:
 - made separate phase and magnitude file selection buttons to prevent confusion on the order of selection
25-04-17:
 - moved opening undistorted file to GUI part in startProcessing()
19-04-17: 
 - changed getfilename functions to not invoke plugin but use BrowseFile()
 - set "StartFile" to "PhaseFile"
02-01-17: added pushbutton, radiobuttons, getFileNamesFile() for batch
03-03-14: 
- in unit calculations, remove `partial Fourier' and replace with GRAPPA/SENSE `acceleration factor'; adapt calculation (from line 780)
- line 731: replace `rows in image' by 'matrix size y' 
19-12-13: added getFileType(filestage, fileid)  
18-12-13: added getVMRFile() function, getFileExtension(), getFileTypeFromExtension()
05-12-13: added VMR button and label
06-06-12: switched checkerrorstatus() off at end of procedure
05-10-11: closed and re-opened FMRs after Evaluation
03-10-11: added extendFileName() function
07-06-11: added chbPhaseToRadians, chbMaskLaplacian for unwrapping
28-09-10: removed echospacing widgets, debugging.
23-09-10: upgraded to BrainVoyager QX 2.2

Hester Breman, Brain Innovation, 2010-2018
*/

var scriptObj 					= new Object; 
scriptObj.scriptNameForUser 	= "anatabacus"; 
scriptObj.dialogFileName 		= "anatabacus_v120.ui"; 
scriptObj.dialogAccessName      = "anatabacusDlg"; 
BrainVoyager.PrintToLog("anatabacus dialog started...");

// filetype (FMR or NIFTI)
const UNKNOWN_FILE_TYPE     = 0;
const BV_TYPE_VMR           = 20;
const BV_TYPE_FMR           = 21;
const NIFTI_FILE_TYPE       = 1; 
const NIFTI_PAIR_FILE_TYPE  = 2;

// stage of file(s) to be collected
const FILE_STAGE_NOT_SET                    = 0;
const FILE_STAGE_PHASE_UNWRAP               = 1; // unwrap
const FILE_STAGE_PHASE_COMPUTE_B0           = 2; // calculate field inhomogeneity
const FILE_STAGE_PHASE_COMPUTE_PIXSH        = 3; // calculate pixelshift
const FILE_STAGE_EPI_UNDISTORT              = 4; // undistort
const FILE_STAGE_EPI_REQUIRED               = 5; // any step requiring EPI
const FILE_STAGE_EPI_UNDISTORT_WITH_COREG   = 6;

// conversions
const CONVERT_OPTION_UNKNOWN = 0;
const CONVERT_SI_UNIT_MIN2SEC = 1;
const CONVERT_SI_UNIT_MSEC2SEC = 2;
const CONVERT_SI_UNIT_MICROSEC2SEC = 3;
const CONVERT_SI_UNIT_NANOSEC2SEC = 4;
const CONVERT_SI_UNIT_MM2M = 5;
const CONVERT_SI_UNIT_CM2M = 6;
const CONVERT_SI_UNIT_KHZ2HZ = 7;
const CONVERT_SI_UNIT_MHZ2HZ = 8;
const CONVERT_SI_UNIT_GHZ2HZ = 9;
const CONVERT_CART2POL = 10;
const CONVERT_POL2CART = 11;
const CONVERT_DEG2RAD = 12;
const CONVERT_RAD2DEG = 13;
const CONVERT_RAD2HZ = 14;
const CONVERT_PPM2HZ = 15;
const CONVERT_ESP2BWPE = 16;

// global variables
var cancel = false;
var nrofcycles = 0;
var nrofiterations = 0;
var do3DFMG = true;
var nrofiterationsrelax = 0;
var regularisation = 0.0;
var phase2radians = true;
var masklaplacian = false;
var bandwidthpe = 0;
var echotimediff = 0.0;
var phasencodingdir = 0;
var apply2pi = false;
var startfileid, epifileid;
var projfilenames, nroffiles; // new v1.2.0


// widgets tab 1: Distortion correction
var startStepDDBox, thisStepRadioButton, allStepsRadioButton, chbPhaseToRadians;
var bandwidthBox, echotimediffBox, encdirBox, apply2piBox, chbCoregField2Func; 
var magnFileLabel, phaseFileLabel, unwrFileLabel, b0FileLabel, pixshiftFileLabel, epiFileLabel, vmrFileLabel, vmrFileButton;
var magnFileButton, phaseFileButton, unwrappedFileButton, b0FileButton, pixshiftFileButton, epiFileButton, proceedButton, startButton;
var fmrRadioButton;//, niiRadioButton, hdrRadioButton;
var btnLoadText, rbtnLoadText, rbtnFilesGUI, lblTxtFile; // new v1.2.0
// widgets tab 2: Unwrapping options
var rbtn3DFMG, rbtnRelaxation, spbNrIterRelaxUnwrap, dspbRegularisation;
var spbNrVCycles, spbIterPerCycle, chbMaskLaplacian;
// widgets tab 3: Evaluation
var btnGetDistImg, btnGetUndistImg, lblGetDistImg, lblGetUndistImg, btnCreateDiffImg;
var distfmr, undistfmr;
// widgets tab 4: Conversions
var input1Label, input2Label, input3Label, outputField, outputLabel, dspbInput1, dspbInput2, dspbInput3;
var rbtnDeg2Rad, rbtnRad2Deg, rbtnRad2Hz, rbtnPPM2Hz, rbtnES2PEBW, rbtnDFreq2Dx;
var btnConvertOtherCalc;
var btnConvertCart2Pol, btnConvertPol2Cart, leRealCoord, leImagCoord, leMagn, lePhase;
var fieldSIamount, boxSIamountTime, boxSIamountDistance, boxSIamountFrequency, btnConvertSIunits, secondsField, metersField, hertzField;
var btnGetRealFileName, btnGetImagFileName, btnConvertCartFiles2PolFiles, lblRealFileName, lblImagFileName; 
//var , btnConvertPPMFile2Hz, btnGetPPMFileName;*/


// check parameters
var startstep = FILE_STAGE_NOT_SET;
var unwrappingparamspresent = false;
var acqparamspresent = false;
var datatoselect = 0;
var procedure = "Unknown";
var docoreg = false;
var doAll = false;


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 1: Distortion correction ~~~~~~~~~~~~~~~~~~~~

scriptObj.proceedToFileUpload = function() {

    var proceed = true;
    
    // read start step and acquisition parameters
    startstep = startStepDDBox.currentIndex;
    
    // get nr of steps
    doAll = allStepsRadioButton.checked; 
    
    switch (startstep) { 
        case FILE_STAGE_PHASE_UNWRAP:  
            procedure = "Unwrap";
            //phaseMagnFileButton.enabled = true;
            magnFileButton.enabled = true;
            phaseFileButton.enabled = true;
            datatoselect = 2; // magnitude + phase
            if (doAll) {
                  epiFileButton.enabled = true; 
                  datatoselect += 1;
             if (chbCoregField2Func.checked) {
                    vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                    datatoselect += 1;
                }              
            } 
            break; 
        case FILE_STAGE_PHASE_COMPUTE_B0: 
            procedure = "ComputeB0";
            unwrappedFileButton.enabled = true;
            datatoselect = 1;
            if (doAll) {
                  epiFileButton.enabled = true; 
                  datatoselect += 1;
                 if (chbCoregField2Func.checked) {
                    vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                    datatoselect += 1;
                }                    
            }            
            break;
        case FILE_STAGE_PHASE_COMPUTE_PIXSH:
            procedure = "ComputePxshift";
            b0FileButton.enabled = true;
            datatoselect = 1;
            if (doAll) {
                epiFileButton.enabled = true; 
                datatoselect += 1;
                 if (chbCoregField2Func.checked) {
                    vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                    datatoselect += 1;
                }                  
            }            
            break;
        case FILE_STAGE_EPI_UNDISTORT: 
            procedure = "Undistort";
            pixshiftFileButton.enabled = true;
            epiFileButton.enabled = true; 
            datatoselect = 2; // EPI + pixelshift
           if (chbCoregField2Func.checked) {
                vmrFileButton.enabled = true; // 18-12-13: now try via GUI signals/slots         
                datatoselect += 1;
            } 
            break;
        default:   
            proceed = false;
            BrainVoyager.PrintToLog("No startstep selected. Please try again.");
    }
    
   // enable the radiobuttons
    if (proceed) {
            fmrRadioButton.enabled = true;

        if (startstep == FILE_STAGE_PHASE_UNWRAP) {
            var unwrparamsok = this.collectUnwrappingParameters();
        }
        proceed = this.collectAcquisitionParameters();
    }

}

/*
scriptObj.getFiles = function(filestage){

    var success = true;
    var fileid = this.getFileType();
BrainVoyager.PrintToLog("fileid: " + fileid);    
    return this.getFiles(filestage, fileid);
}*/
 
scriptObj.getFiles = function(filestage, fileid) {

    var success = true;
    if (fileid == undefined) {
//    var fileid = this.getFileType(); // >>> is niet goed, vraagt alleen FMR or NIfTI.
        fileid = this.getFileType();  
    }
  //  var fileid = this.getFileTypeFromExtension(); // 181213; lekker slim, je weet de filenaam nog niet!
    BrainVoyager.SetPluginIntParameter("FileTypeId", fileid); // FMR, VMR or NIfTI; to add: DMR?
//    if (fileid != BV_TYPE_VMR) {startfileid = fileid;
  //  }
    BrainVoyager.SetPluginIntParameter("FileStage", filestage);
    BrainVoyager.SetPluginStringParameter("Command", "GetFiles"); 
    BrainVoyager.ExecutePlugin(); 
        // now as many files can be selected as possible
        // writes also a text file to disk with the names
    var filenamessize = BrainVoyager.GetPluginIntParameter("NrOfFiles");
    BrainVoyager.PrintToLog("Number of selected files: " + filenamessize);
    if (filenamessize == 0) {
            BrainVoyager.PrintToLog("No files selected. Procedure canceled.");
            cancel = true;
            success = false;
    } else {
        datatoselect -= filenamessize;
    }
    return success;
}

scriptObj.getMagnFile = function() {
	BrainVoyager.PrintToLog("Get magnitude file...");
    var magnitudename = BrainVoyager.BrowseFile("Please select the magnitude file", "*.fmr");
        if ((magnitudename.length > 0)) {
        BrainVoyager.OpenDocument(magnitudename);
      //  filename = BrainVoyager.GetPluginStringParameter("MagnFile"); //// changed to this 06-12-10
        magnFileLabel.text = magnitudename;
        BrainVoyager.SetPluginStringParameter("MagnFile", magnitudename);
    }
}

scriptObj.getPhaseFile = function() {
    BrainVoyager.PrintToLog("Get wrapped phase file...");
    //var success = this.getFiles(FILE_STAGE_PHASE_UNWRAP);
  //  var magnitudename = BrainVoyager.BrowseFile("Please select the magnitude file", "*.fmr");
    var phasename = BrainVoyager.BrowseFile("Please select the phase file", "*.fmr");
    //    if (success) {
    if (phasename.length > 0) {

        BrainVoyager.OpenDocument(phasename);
        phaseFileLabel.text = phasename;
        BrainVoyager.SetPluginStringParameter("PhaseFile", phasename);
    }
}

scriptObj.getUnwrappedFiles = function() {
	BrainVoyager.PrintToLog("Get unwrapped phase map(s)..."); 
   // var success = this.getFiles(FILE_STAGE_PHASE_COMPUTE_B0);
    var unwrappedphasename = BrainVoyager.BrowseFile("Please select the unwrapped phase file", "*.fmr");
   if (unwrappedphasename.length > 0) {
     //   var filename = BrainVoyager.GetPluginStringParameter("UnwrPhaseFile");
        unwrFileLabel.text = unwrappedphasename;
     //   if (startfileid == BV_TYPE_FMR)
        BrainVoyager.OpenDocument(unwrappedphasename);
        BrainVoyager.SetPluginStringParameter("PhaseFile", phasename);
    }
}

scriptObj.getB0Files = function() {
	BrainVoyager.PrintToLog("Get field inhomogeneity map(s)..."); 
  //  var success = this.getFiles(FILE_STAGE_PHASE_COMPUTE_PIXSH);
    var b0name = BrainVoyager.BrowseFile("Please select the field inhomogeneity (B0) file", "*.fmr");
//    if (success) {
     if (b0name.length > 0) {
      //  var filename = BrainVoyager.GetPluginStringParameter("B0File");
        b0FileLabel.text = b0name;
        //if (startfileid == BV_TYPE_FMR)
        BrainVoyager.OpenDocument(b0name);
        BrainVoyager.SetPluginStringParameter("PhaseFile", phasename);
    }
}

scriptObj.getPixelshiftFiles = function() {
	BrainVoyager.PrintToLog("Get pixel shift map(s)..."); 
  //  var success = this.getFiles(FILE_STAGE_EPI_UNDISTORT);
    var pixelshiftname = BrainVoyager.BrowseFile("Please select the pixelshift file", "*.fmr");
 //   if (success) {
     if (pixelshiftname.length > 0) {
       // var filename = BrainVoyager.GetPluginStringParameter("PixshiftFile");
        pixshiftFileLabel.text = pixelshiftname;
        //if (startfileid == BV_TYPE_FMR)
        BrainVoyager.OpenDocument(pixelshiftname);
        BrainVoyager.SetPluginStringParameter("PhaseFile", phasename);
    }
}

scriptObj.getEPIFiles = function() { 

	BrainVoyager.PrintToLog("Get EPI data..."); 
  //  var success = this.getFiles(FILE_STAGE_EPI_REQUIRED);
    var epiname = BrainVoyager.BrowseFile("Please select the EPI file", "*.fmr");
  //  if (success) {
     if (epiname.length > 0) {
  //      var filename = BrainVoyager.GetPluginStringParameter("EPIFile");
        epiFileLabel.text = epiname;
        //if (startfileid == BV_TYPE_FMR)
        BrainVoyager.OpenDocument(epiname);
        BrainVoyager.SetPluginStringParameter("EPIFile", epiname);
    }
}

scriptObj.getVMRFile = function() { 

	BrainVoyager.PrintToLog("Get VMR data..."); 
  //  var success = this.getFiles(FILE_STAGE_EPI_UNDISTORT_WITH_COREG, BV_TYPE_VMR);
    var vmrname = BrainVoyager.BrowseFile("Please select the VMR file", "*.vmr");
  //  if (success) {
     if (vmrname.length > 0) {
    //    var vmrname = BrainVoyager.GetPluginStringParameter("VMRFile");
        vmrFileLabel.text = vmrname;
        BrainVoyager.OpenDocument(vmrname);
        BrainVoyager.SetPluginStringParameter("VMRFile", vmrname);
    }
}

scriptObj.getFileType = function() {

    var filetype;  
    if (fmrRadioButton.checked == 1) {
        filetype = BV_TYPE_FMR; 
  //  } else if (niiRadioButton.checked == 1) {
  //      filetype = NIFTI_FILE_TYPE;
  //  } else if (hdrRadioButton.checked == 1) {
   //     filetype = NIFTI_PAIR_FILE_TYPE;
    }     
    return filetype;
}

scriptObj.getFileNamesFile = function() {

    var filenamesfile = BrainVoyager.BrowseFile("Please select the file with filenames", "*.txt");
    lblTxtFile.text = filenamesfile;
    BrainVoyager.PrintToLog("Selected " + filenamesfile);
//BrainVoyager.PrintToLog("In getFileNamesFile, before reading file");
    this.readTextFileIntoProjectNamesArrayAllStages(filenamesfile);
}

    /*
     
     >> see diagram
     
     const FILE_STAGE_PHASE_UNWRAP               = 1; // unwrap
     const FILE_STAGE_PHASE_COMPUTE_B0           = 2; // calculate field inhomogeneity
     const FILE_STAGE_PHASE_COMPUTE_PIXSH        = 3; // calculate pixelshift
     const FILE_STAGE_EPI_UNDISTORT              = 4; // undistort
     const FILE_STAGE_EPI_REQUIRED               = 5; // any step requiring EPI
     const FILE_STAGE_EPI_UNDISTORT_WITH_COREG   = 6;
     */

scriptObj.readTextFileIntoProjectNamesArrayAllStages = function(textfilename) {

        /* how about: n subjects x
         0. magnitude FMR (or none)
         1. phase FMR (or none)
         2. b0 FMR (or none)
         3. pixelshift FMR (or none)
         4. epi to undistort FMR
         5. vmr
         */
        var filecounter;
        var success = true;
   //     var textfilename = BrainVoyager.BrowseFile("Please select the text file with filenames", "*.txt");
        BrainVoyager.PrintToLog("Start reading: " + textfilename);
        var name = String(textfilename); // circumvention of problem with implicit strings
        var filenamesfile = new QFile(name);
        filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
        var textstr = new QTextStream(filenamesfile);
        nroffiles = parseInt(textstr.readLine());
        BrainVoyager.PrintToLog("Number of projects: " + nroffiles);
        projfilenames = new Array(nroffiles);// projfilenames has not yet been used in the process
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            textstr.readLine(); // empty line
            filesPerProject = new Array(6);
            var projfilecounter;
            for (projfilecounter = 0; projfilecounter < 6; projfilecounter++) {
                var filename = textstr.readLine();
                BrainVoyager.PrintToLog("Project: " + (filecounter+1) + ", file " + (projfilecounter+1) + ":\n" + filename);
                filesPerProject[projfilecounter] = filename;
            }
            projfilenames[filecounter] = filesPerProject;
        }
        filenamesfile.close();
        
}

scriptObj.writeProjectNamesToTextFileAllStages = function(projfilenames, stagedescr) {
    
    var filecounter, projfilecounter;
    var success;
    
    // write the textfile to the path of the first file
    var name = String(this.getPath(projfilenames[0][0]) + "filenames_for_anatabacus_stage_" + stagedescr + "_v120.txt");
    BrainVoyager.PrintToLog("Start writing file: " + name);
    var filenamesfile = new QFile(name);
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly));
    var textstr = new QTextStream(filenamesfile);
    textstr.writeDouble(projfilenames.length);
    for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        textstr.writeString("\n");
        for (projfilecounter = 0; projfilecounter < 6; projfilecounter++) {
            textstr.writeString(projfilenames[filecounter][projfilecounter] + "\n");
        }
    }
    filenamesfile.close();
    BrainVoyager.PrintToLog("Wrote " + name + ".");
    return success;
}

scriptObj.getFileExtension = function(filename) {

    var posOfDot = filename.lastIndexOf(".");    
    var extension = filename.substring((posOfDot+1), filename.length);
    return extension;
}


scriptObj.getFileTypeFromExtension = function(filename) {

    var filetype;    
    var ext = this.getFileExtension(filename);
        
    if (filename.indexOf("fmr") > -1) {
        filetype = BV_TYPE_FMR; 
    } else if (filename.indexOf("vmr") > -1) {
        filetype = BV_TYPE_VMR;     
    } else if (filename.indexOf("nii") > -1) {
        filetype = NIFTI_FILE_TYPE;
    } else if (filename.indexOf("hdr") > -1) {
        filetype = NIFTI_PAIR_FILE_TYPE;
    } 
    return filetype;
}


scriptObj.collectUnwrappingParameters = function() {

    var success = true;
    BrainVoyager.PrintToLog("Unwrapping parameters:"); 
    
    if (rbtnRelaxation.checked == 1) {
    
        do3DFMG = false;
        nrofiterationsrelax = spbNrIterRelaxUnwrap.value;
        regularisation = dspbRegularisation.value;
        BrainVoyager.PrintToLog("Number of iterations for relaxation: " + nrofiterationsrelax);
        BrainVoyager.PrintToLog("Regularisation parameter: " + regularisation);
        
    } else {
    
        nrofcycles = spbNrVCycles.value;
        nrofiterations = spbIterPerCycle.value;
        masklaplacian = chbMaskLaplacian.checked;
        BrainVoyager.PrintToLog("Number of V-Cycles for unwrapping: " + nrofcycles); 
        BrainVoyager.PrintToLog("Number of iterations: " + nrofiterations); 
        BrainVoyager.PrintToLog("Convert phase data to radians (-pi to pi) before unwrapping: " + (phase2radians ? "yes" : "no")); 
        BrainVoyager.PrintToLog("Use mask for Laplacian to reduce influence of noise: " + (masklaplacian ? "yes" : "no"));  
    }  
    
    // check parameters
    if (do3DFMG) {
        if (nrofcycles == 0 || nrofiterations == 0) {
            BrainVoyager.PrintToLog("One or more of the unwrapping parameters is 0. Unwrapping results may be suboptimal.");
            success = false;
        } else {
            unwrappingparamspresent = true;
        }
    } else {
        if (nrofiterationsrelax == 0) {
            BrainVoyager.PrintToLog("Number of iterations for unwrapping is 0. Unwrapping results may be suboptimal.");
            success = false;
        } else {
            unwrappingparamspresent = true;
        }
    }
    
    if (!phase2radians){
        BrainVoyager.PrintToLog("Please note that the conversion to radians, which is necessary in most cases, is switched off.")
    } 
    return success;
}

scriptObj.collectAcquisitionParameters = function() { 

      var success = true;
      BrainVoyager.PrintToLog("Acquisition parameters:"); 
      bandwidthpe = bandwidthBox.value;
      echotimediff = echotimediffBox.value;
      phasencodingdir = encdirBox.currentIndex;
      // here is a circumvention because other directions have been removed (25-10-11)      
      if (phasencodingdir == 1) { // +y
        phasencodingdir = 3;
      } else if (phasencodingdir == 2) { // -y
        phasencodingdir = 4;
      }
      
      apply2pi = false;
      BrainVoyager.PrintToLog("Bandwidth in phase encoding direction: " + bandwidthpe);
      BrainVoyager.PrintToLog("Echo time difference (TE2 - TE1): " + echotimediff);
      BrainVoyager.PrintToLog("Phase encoding direction, text: " + encdirBox.currentText + " (" + phasencodingdir + ")"); 
      
      // check parameters
      if (bandwidthpe == 0 || echotimediff == 0.0 || phasencodingdir == 0) {
        BrainVoyager.PrintToLog("One or more of the acquisition parameters is 0");
        success = false;
      } else {
        acqparamspresent = true;
      }
      phase2radians = chbPhaseToRadians.checked;
      BrainVoyager.PrintToLog("Convert phase data to radians: " + (phase2radians ? "yes" : "no"));
      docoreg = chbCoregField2Func.checked; // 11-12-13
      BrainVoyager.PrintToLog("Register VDM to EPI: " + (docoreg ? "yes" : "no"));
      return success;
}

scriptObj.closeAllProjects = function() {
    
    var pn = BrainVoyager.PrintToLog;
    BrainVoyager.PrintToLog("Closing all projects...");
    var docs = BrainVoyager.Documents;
    var n_docs = docs.Count;
    if (n_docs == 0)
        return;
    var i;
    while (n_docs > 0)
    {
        BrainVoyager.ActiveDocument.Close(); // deze werkt in ieder geval
        docs = BrainVoyager.Documents;
        n_docs = docs.Count;
    }
}

scriptObj.startProcessing = function() {

    BrainVoyager.PrintToLog("Start processing..."); 
    var filespresent = true;
  //  if ((procedure != "Unknown") && (startstep > 0) && (datatoselect == 0)) {
    if ((procedure != "Unknown") && (startstep > 0)) {
        
        // first send all parameters to anatabacus
        BrainVoyager.SetPluginStringParameter("Command", "Start");
        BrainVoyager.SetPluginStringParameter("Procedure", procedure);
        BrainVoyager.SetPluginIntParameter("PhaseToRadians", (phase2radians ? 1 : 0)); // 07-06-11
        BrainVoyager.SetPluginIntParameter("UnwrappingMethod", (do3DFMG ? 1 : 0));
        if (do3DFMG) {
            BrainVoyager.SetPluginIntParameter("NrOfVCycles", nrofcycles); // for unwrapping, 21-02-11
            BrainVoyager.SetPluginIntParameter("NrOfIterations", nrofiterations); // for unwrapping, 21-02-11
            BrainVoyager.SetPluginIntParameter("MaskLaplacian", (masklaplacian ? 1 : 0)); // 07-06-11
        } else {
            BrainVoyager.SetPluginIntParameter("NrOfIterationsRelax", nrofiterationsrelax); // 11-10-11
            BrainVoyager.SetPluginFloatParameter("Regularisation", regularisation);
        }
        
        BrainVoyager.SetPluginFloatParameter("BandWidthPE", bandwidthpe);
        BrainVoyager.SetPluginFloatParameter("EchoTimeDiff", echotimediff);
        BrainVoyager.SetPluginIntParameter("PhaseEncodingDir", phasencodingdir);
        BrainVoyager.SetPluginIntParameter("ApplyTwoPi", (apply2pi ? 1 : 0)); // 19-07-11
        BrainVoyager.SetPluginIntParameter("PerformAllSteps", (doAll ? 1 : 0));
        BrainVoyager.SetPluginIntParameter("StartFileTypeId", startfileid); // FMR, (DMR) or NIfTI
        
        
        // now deal with the filenames
 //       if (((startstep == FILE_STAGE_PHASE_UNWRAP) && (!doAll) || acqparamspresent)) { // is set in collectAcquisitionParameters
        if (acqparamspresent) {
            
            var startfile  = "";//021018
            var epifile = "";
            if (rbtnFilesGUI.checked) {
                
                // get filenames from textlabels (this is redundant, will be changed in later version)
                // (you could save them in projfilenames instead)
                switch (startstep) {
                    case FILE_STAGE_PHASE_UNWRAP:               startfile = phaseFileLabel.text;    break;
                    case FILE_STAGE_PHASE_COMPUTE_B0:           startfile = unwrFileLabel.text;     break;
                    case FILE_STAGE_PHASE_COMPUTE_PIXSH:        startfile = b0FileLabel.text;       break;
                    case FILE_STAGE_EPI_UNDISTORT:              startfile = pixshiftFileLabel.text; break;
                }
                if (startfile.length < 1) filespresent = false;
                BrainVoyager.SetPluginStringParameter("PhaseFile", startfile);
                if (startstep == FILE_STAGE_PHASE_UNWRAP) {
                    var magnfilename = magnFileLabel.text;
                    if (magnFileLabel.text.length < 1) {
                        filespresent = false;
                    } else {
                        BrainVoyager.SetPluginStringParameter("MagnFile", magnFileLabel.text);
                    }
                }
                if ((doAll) || (startstep == FILE_STAGE_EPI_UNDISTORT)) {
                    BrainVoyager.SetPluginIntParameter("mapFieldmapToFunctional", chbCoregField2Func.checked);
                    if (chbCoregField2Func.checked) {
                        var vmrname = vmrFileLabel.text;
                        if (vmrname.length < 1) {
                            filespresent = false;
                        } else {
                            BrainVoyager.SetPluginStringParameter("VMRFile", vmrFileLabel.text);
                        }
                    }
                    epifile = epiFileLabel.text;
                    if (epifile.length < 1) {
                        filespresent = false;
                    } else {
                        BrainVoyager.SetPluginStringParameter("EPIFile", epifile);
                        BrainVoyager.SetPluginIntParameter("EPIFileTypeId", startfileid); // FMR or NIfTI
                    }
                }
                
                if (filespresent) {
                    // now start processing
                    this.anatabacusDlg.close();
                    BrainVoyager.ExecutePlugin();
                    
                    // ask parameters from plugin
                    if ((this.checkerrorstatus() > -1) && ((doAll) || (startstep == FILE_STAGE_EPI_UNDISTORT))) {
                        
                        var distortedFMR = BrainVoyager.ActiveDocument; // because the EPI is the last selected image, so is the active document
                        distortedFMR.Close();
                        //     var undistortedname = BrainVoyager.GetPluginStringParameter("NameUndistortedFile"); // this is set in undistort1d() in plugin (without path)
                        var undistortedname = extendFileName(epifile, "undistort");
                        BrainVoyager.PrintToLog("Opening " + undistortedname + "...");
                        var undistortedFMR = BrainVoyager.OpenDocument(undistortedname.toString());
                    }
                    
                } else {
                    BrainVoyager.PrintToLog("Files missing in GUI part.");
                }
                
            } else { // get files via text file and repeatedly invoke plugin

                var projcounter;
                this.anatabacusDlg.close(); // dit werkt niet
                for (projcounter = 0; projcounter < nroffiles; projcounter++) {
                    
                    switch (startstep) {
                        case FILE_STAGE_PHASE_UNWRAP:               startfile = projfilenames[projcounter][1]; break;
                        case FILE_STAGE_PHASE_COMPUTE_B0:           startfile = projfilenames[projcounter][2]; break;
                        case FILE_STAGE_PHASE_COMPUTE_PIXSH:        startfile = projfilenames[projcounter][3]; break;
                        case FILE_STAGE_EPI_UNDISTORT:              startfile = projfilenames[projcounter][4]; break;
                    }
                    BrainVoyager.OpenDocument(startfile);
                    BrainVoyager.SetPluginStringParameter("PhaseFile", startfile);
                    if (startstep == FILE_STAGE_PHASE_UNWRAP) {
                        BrainVoyager.OpenDocument(projfilenames[projcounter][0]);
                        BrainVoyager.SetPluginStringParameter("MagnFile", projfilenames[projcounter][0]);
                    }
                    if ((doAll) || (startstep == FILE_STAGE_EPI_UNDISTORT)) {
                        BrainVoyager.SetPluginIntParameter("mapFieldmapToFunctional", chbCoregField2Func.checked);
                        if (chbCoregField2Func.checked) {
                            BrainVoyager.OpenDocument(projfilenames[projcounter][5]);
                            BrainVoyager.SetPluginStringParameter("VMRFile", projfilenames[projcounter][5]);
                        }
                        epifile = projfilenames[projcounter][4];
                        BrainVoyager.OpenDocument(epifile);
                        BrainVoyager.SetPluginStringParameter("EPIFile", epifile);
                        BrainVoyager.SetPluginIntParameter("EPIFileTypeId", startfileid); // FMR or NIfTI
                    }
                    
                    // now start processing
                    BrainVoyager.ExecutePlugin();
                    
                    this.closeAllProjects();
                }
            }
            
        } else {
            BrainVoyager.PrintToLog("Missing acquisition parameters.");
        }
    
    } else {
        BrainVoyager.PrintToLog("Procedure not selected or missing filenames.");
    }
     
    // get feedback
//    var errornr = this.checkerrorstatus(); // prints error message to log tab, if there is one    
   // BrainVoyager.PrintToLog("~~ closing anatabacus ~~");
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 2: Evaluation ~~~~~~~~~~~~~~~~~~~~

scriptObj.getDistortedImageFile = function() {
	BrainVoyager.PrintToLog("Get distorted image file..."); 
    var filename = BrainVoyager.BrowseFile("Please select the distorted image file", "*.fmr");//")this.getFile(BV_TYPE_FMR);
    if (filename.length > 0) {
        lblGetDistImg.text = filename;
        distfmr = BrainVoyager.OpenDocument(filename); 
    }
}

scriptObj.getUndistortedImageFile = function() {
	BrainVoyager.PrintToLog("Get undistorted image file..."); 
    //var filename = this.getFile(BV_TYPE_FMR);
    var filename = BrainVoyager.BrowseFile("Please select the undistorted image file", "*.fmr");
    if (filename.length > 0) {
        lblGetUndistImg.text = filename;
        undistfmr = BrainVoyager.OpenDocument(filename);
    }
}

scriptObj.createDifferenceImage = function() {

    BrainVoyager.PrintToLog("difference image... ");
    var distortedFile = lblGetDistImg.text; // these images have been opened in BrainVoyager after file selection
    var undistortedFile = lblGetUndistImg.text;
    if ((distortedFile.length > 1) && (undistortedFile.length > 1)) {
    
        // create difference image
        BrainVoyager.PrintToLog("Create difference image of \n" + distortedFile + " and \n" + undistortedFile);
        BrainVoyager.SetPluginStringParameter("Command", "Evaluate"); 
        BrainVoyager.ExecutePlugin();    
        this.anatabacusDlg.close();
        distfmr.Close();
        undistfmr.Close();        
        
        var errornr = this.checkerrorstatus();
        if (errornr > -1) { 
            // show results
            var diffimgfmr = BrainVoyager.GetPluginStringParameter("DifferenceImgName");
            BrainVoyager.OpenDocument(diffimgfmr); 
            distfmr = BrainVoyager.OpenDocument(distortedFile);
            undistfmr = BrainVoyager.OpenDocument(undistortedFile);        
        }
    } else {
        BrainVoyager.PrintToLog("Invalid file names for creating difference image.");
    }
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 3: Conversions ~~~~~~~~~~~~~~~~~~~~


scriptObj.getRealFileName = function() {
	BrainVoyager.PrintToLog("Get real file..."); 
  //  var success = this.getFile(BV_TYPE_FMR); //???? found like this 19-04-17
    var filename = BrainVoyager.BrowseFile("Please select the real file", "*.fmr");
 //   if (success) {
    if (filename.length > 0) {
 //       var filename = BrainVoyager.GetPluginStringParameter("FileName");
        lblRealFileName.text = filename;
        BrainVoyager.OpenDocument(filename); 
    }
}

scriptObj.getImaginaryFileName = function() {
	BrainVoyager.PrintToLog("Get imaginary file..."); 
 //   var success = this.getFile(BV_TYPE_FMR); //???? found like this 19-04-17
    var filename = BrainVoyager.BrowseFile("Please select the imaginary file", "*.fmr");
  //  if (success) {
    if (filename.length > 0) {
      //  var filename = BrainVoyager.GetPluginStringParameter("FileName");
        lblImagFileName.text = filename;
        BrainVoyager.OpenDocument(filename); 
    }
}

scriptObj.getPPMFileName = function() {
	BrainVoyager.PrintToLog("Get parts-per-million file..."); 
 //   var success = this.getFile(BV_TYPE_FMR);
    var filename = BrainVoyager.BrowseFile("Please select the parts-per-million file", "*.fmr");
   // if (success) {
    if (filename.length > 0) {
    //    var filename = BrainVoyager.GetPluginStringParameter("FileName");
        lblPPMfileName.text = filename;
        //if (startfileid == BV_TYPE_FMR)
        BrainVoyager.OpenDocument(filename);
    }
}


scriptObj.convertSIunits = function() {

    var siAmount = fieldSIamount.text;
    var selectedSIquantity;
    
    if (!isNaN(siAmount)) {
    
    // time
    if (boxSIamountTime.currentIndex > 0) {
        selectedSIquantity = boxSIamountTime.currentText;
        var quantityInSec;
        if (selectedSIquantity == "minutes") {
            quantityInSec = (siAmount * 60);
        } else if (selectedSIquantity == "milliseconds") {
            quantityInSec = (siAmount * Math.pow(10,-3));
        } else if (selectedSIquantity == "microseconds") {
            quantityInSec = (siAmount * Math.pow(10,-6));
        } else if (selectedSIquantity == "nanoseconds") {
            quantityInSec = (siAmount * Math.pow(10,-9));
        }
        secondsField.text = quantityInSec;
    }
    
    // position
    if (boxSIamountDistance.currentIndex > 0) {
        selectedSIquantity = boxSIamountDistance.currentText;
        var quantityInMeters;
        if (selectedSIquantity == "millimeter") {
            quantityInMeters = (siAmount * Math.pow(10,-3));
        } else if (selectedSIquantity == "centimeter") {
            quantityInMeters = (siAmount / 100);
        }    
        metersField.text = quantityInMeters;  
    }
    
    if (boxSIamountFrequency.currentIndex > 0) {
        selectedSIquantity = boxSIamountFrequency.currentText;
        var quantityInHz;
        if (selectedSIquantity == "kilohertz") {
            quantityInHz = (siAmount * Math.pow(10,3));
        } else if (selectedSIquantity == "megahertz") {
            quantityInHz = (siAmount * Math.pow(10,6));
        }  else if (selectedSIquantity == "gigahertz") {
            quantityInHz = (siAmount * Math.pow(10,9));
        }         
        hertzField.text = quantityInHz;
    }

    } else {
        BrainVoyager.PrintToLog("Convert to SI units: input value(s) not-a-number (NaN).");
    }
}

scriptObj.convertCartesianToPolar = function() {

    BrainVoyager.PrintToLog("Convert cartesian to polar coordinates...");
    var isPolar = false;    
    var realval = leRealCoord.text;
    var imagval = leImagCoord.text;
    var z = new Complex(realval, imagval, isPolar); 
    if (!isNaN(realval) && !isNaN(imagval)) { 
        leMagn.text = z.magnitude().toFixed(4);
        lePhase.text = z.phase().toFixed(4); 
    } else {
        BrainVoyager.PrintToLog("Convert cartesian to polar: input value(s) not-a-number (NaN).");  
    }
}

scriptObj.convertPolarToCartesian = function() { 

    var isPolar = true;
    var magnitude = leMagn.text;
    var phase = lePhase.text; 
    var z = new Complex(magnitude, phase, isPolar);
    if (!isNaN(magnitude) && !isNaN(phase)) { 
            leRealCoord.text = z.real.toFixed(4); 
            leImagCoord.text = z.imag.toFixed(4); 
    } else {
        BrainVoyager.PrintToLog("Convert polar to cartesian: input value(s) not-a-number (NaN).");  
    }
}

scriptObj.convertCartesianToPolarFiles = function() {

    var realfile = lblRealFileName.text;
    var imagfile = lblImagFileName.text;
    
  //     var success = true;
    BrainVoyager.SetPluginStringParameter("Command", "CreatePolarFile"); 
    BrainVoyager.SetPluginStringParameter("RealFile", realfile);
    BrainVoyager.SetPluginStringParameter("ImagFile", imagfile);
    
    this.anatabacusDlg.close();  
    BrainVoyager.ExecutePlugin(); 
  //  var filename = BrainVoyager.GetPluginStringParameter("FileName");
//BrainVoyager.PrintToLog("Received filename: " + filename);    
    
//    return filename;
  
    
}

scriptObj.convertPPMFile2Hz = function() {

}
   
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ functions tab 3: conversions: other calculations ~~~~~~~~~~~~~~~~~~~~ 

scriptObj.clearOtherCalcFields = function() {

    dspbInput1.value = 0.0; 
    dspbInput2.value = 0.0; 
    dspbInput3.value = 0.0;
    outputField.text = "";
}

scriptObj.setDeg2RadLabel = function() {

    input1Label.text = "degrees:";// (0-360)
    input2Label.text = "";
    input3Label.text = "";    
    outputLabel.text = "radians:";// (in pi)
    this.clearOtherCalcFields();
}

scriptObj.setRad2DegLabel = function() {

    input1Label.text = "radians:"; // (in pi)
    input2Label.text = "";
    input3Label.text = ""; 
    outputLabel.text = "degrees:";
    this.clearOtherCalcFields();
}

scriptObj.setRad2HzLabel = function() {

    input1Label.text = "radians:";// (in pi)
    input2Label.text = "";
    input3Label.text = "";     
    outputLabel.text = "hertz:";
    this.clearOtherCalcFields();
}

scriptObj.setPPM2HzLabels = function() {

    input1Label.text = "parts per million (ppm):";
    input2Label.text = "field strength (T):";
    input3Label.text = "";//Larmor frequency (Hz)     
    outputLabel.text = "hertz:";
    this.clearOtherCalcFields();
}

scriptObj.setES2PEBWLabels = function() {

    input1Label.text = "echospacing (msec):";
    input2Label.text = "matrix size y:";
 //   input3Label.text = "Fourier encoding (%):";
    input3Label.text = "acceleration factor (phase enc dir):";
    outputLabel.text = "bandwidth (phase enc dir)(Hz):";
    this.clearOtherCalcFields();
}

scriptObj.setDFreq2DxLabels = function() {

    input1Label.text = "bandwidth (phase enc dir)(Hz):";
    input2Label.text = "B0 field inhomogeneity (Hz):";
    input3Label.text = "";
    outputLabel.text = "pixel shift:";
    this.clearOtherCalcFields();
}

scriptObj.convertOtherEntities = function() {


    var param1 = dspbInput1.value; 
    var param2 = dspbInput2.value; 
    var param3 = dspbInput3.value;
    var result = 0.0;
 
    if (rbtnDeg2Rad.checked) {
        // convert degrees to radians
        BrainVoyager.PrintToLog("Convert degrees to radians...");
        result = param1 * (Math.PI / 180);
        
    } else if (rbtnRad2Deg.checked) {
        // convert radians to degrees
        BrainVoyager.PrintToLog("Convert radians to degrees...");
        result = param1 * (180 / Math.PI);
    
    } else if (rbtnRad2Hz.checked) {
        // convert radians to hertz
        BrainVoyager.PrintToLog("Convert radians to Hertz...");
        result = param1 / (2 * Math.PI);
            
    } else if (rbtnPPM2Hz.checked) {
        // convert parts-per-million to hertz
        BrainVoyager.PrintToLog("Convert parts-per-million (ppm) to Hertz (Hz) for 1H...");
        var ppm = param1 * (1/1000000);
        var resonancefreq = 42576000 * param2;
        result = ppm * resonancefreq; 
    
    } else if (rbtnES2PEBW.checked) {
        // convert echospacing to phase encoding bandwidth
        BrainVoyager.PrintToLog("Convert echospacing (esp) to phase encoding bandwidth...");
        var echospacing_sec = param1 * (1/1000); 
        BrainVoyager.PrintToLog("Echospacing in seconds: " + echospacing_sec);
        var ny = param2; 
    /*    var fourierperc = (param3 / 100);  
        BrainVoyager.PrintToLog("Number of phase encoding lines: " + ny + ", Fourier partial scanning: " + fourierperc);
        result = 1 / (echospacing_sec * (ny * fourierperc));  
        BrainVoyager.PrintToLog("bandwidth in phase encoding direction = 1 / (echospacing_sec * (ny * Fourier partial scanning)) = " + result.toPrecision(10));
    */  var accelfac = param3;  
        if ((accelfac == "") || (accelfac == 0)) accelfac = 1;
        var echospacing_sec_acc = echospacing_sec / accelfac; 
        BrainVoyager.PrintToLog("Number of phase encoding lines: " + ny + ", SENSE/GRAPPA acceleration factor in phase encoding direction: " + accelfac);
        result = 1 / (echospacing_sec_acc * ny);  
        BrainVoyager.PrintToLog("bandwidth in phase encoding direction = 1 / (echospacing_sec_acc * ny) = " + result.toPrecision(10));  
    } else if (rbtnDFreq2Dx.checked) {
        // convert frequency shift to spatial shift in pixels
        BrainVoyager.PrintToLog("Convert frequency shift to spatial shift in pixels...");
        
        bandwidth_pe_hz = param1;
        deltab0 = param2;
        result = (deltab0 / bandwidth_pe_hz); 
    }
    outputField.text = result.toPrecision(10);

}


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generic functions ~~~~~~~~~~~~~~~~~~~~ 

scriptObj.getFile = function(fileid) {

    var success = true;
    BrainVoyager.SetPluginIntParameter("FileTypeId", fileid); // FMR or NIfTI
    BrainVoyager.SetPluginStringParameter("Command", "GetFile"); 
    BrainVoyager.ExecutePlugin(); 
    var filename = BrainVoyager.GetPluginStringParameter("FileName");
    return filename;
}



scriptObj.checkerrorstatus = function() {
    var errornr = BrainVoyager.GetPluginIntParameter("ErrorNr");
    if (errornr < 0) {
            var errormsg = BrainVoyager.GetPluginStringParameter("ErrorMsg");
            BrainVoyager.PrintToLog(errormsg);
            cancel = true;
    } 
    return errornr;
}

scriptObj.initDlg = function() { 

    this.anatabacusDlg.windowTitle = "anatabacus"; 
    
    // procedure widgets
    startStepDDBox  = BrainVoyager.FindChild("cmboxStartStep");
    thisStepRadioButton = BrainVoyager.FindChild("rbtnThisStepOnly"); 
    allStepsRadioButton = BrainVoyager.FindChild("rbtnAllNextSteps");
    
    // phase data preparation: convert to radians
    chbPhaseToRadians   = BrainVoyager.FindChild("chbPhaseToRadians");
    
    // phase data preparation: unwrapping parameter widgets
    rbtn3DFMG           = BrainVoyager.FindChild("rbtn3DFMG");
    rbtnRelaxation      = BrainVoyager.FindChild("rbtnRelaxation");
    spbNrIterRelaxUnwrap    = BrainVoyager.FindChild("spbNrIterRelaxUnwrap");
    dspbRegularisation  = BrainVoyager.FindChild("dspbRegularisation"); 
    spbNrVCycles        = BrainVoyager.FindChild("spbNrVCycles");
    spbIterPerCycle     = BrainVoyager.FindChild("spbIterPerCycle");
    chbMaskLaplacian    = BrainVoyager.FindChild("chbMaskLaplacian"); 
    
     // acquisition parameter widgets
    bandwidthBox        = BrainVoyager.FindChild("spnBoxBW");
    echotimediffBox     = BrainVoyager.FindChild("dSpnBoxDTE");
    encdirBox           = BrainVoyager.FindChild("cmBoxPEDir");
    
    // coreg widget
    chbCoregField2Func  = BrainVoyager.FindChild("chbCoregField2Func");

    // file loading widgets
    fmrRadioButton      = BrainVoyager.FindChild("rbtnFMR");
  //  niiRadioButton      = BrainVoyager.FindChild("rbtnNII");
  //  hdrRadioButton      = BrainVoyager.FindChild("rbtnHDR");
    magnFileLabel       = BrainVoyager.FindChild("lblMagnFile");
    phaseFileLabel      = BrainVoyager.FindChild("lblPhaseFile");
    unwrFileLabel       = BrainVoyager.FindChild("lblUnwrFile");
    b0FileLabel         = BrainVoyager.FindChild("lblB0File");
    pixshiftFileLabel   = BrainVoyager.FindChild("lblPixshiftFile");
    epiFileLabel        = BrainVoyager.FindChild("lblEPIfile");
   // phaseMagnFileButton = BrainVoyager.FindChild("btnGetPhaseMagn");
    magnFileButton      = BrainVoyager.FindChild("magnFileButton");
    phaseFileButton     = BrainVoyager.FindChild("phaseFileButton");
    unwrappedFileButton = BrainVoyager.FindChild("btnGetUnwrapped"); 
    b0FileButton        = BrainVoyager.FindChild("btnGetB0");  
    pixshiftFileButton  = BrainVoyager.FindChild("btnGetPixshift"); 
    epiFileButton       = BrainVoyager.FindChild("btnGetEPI"); 
    vmrFileButton       = BrainVoyager.FindChild("btnGetVMR"); 
    vmrFileLabel        = BrainVoyager.FindChild("lblVMRfile");
    btnGetDistImg       = BrainVoyager.FindChild("btnGetDistImg"); 
    btnGetUndistImg     = BrainVoyager.FindChild("btnGetUndistImg"); 
    btnGetRealFileName  = BrainVoyager.FindChild("btnGetRealFileName");
    btnGetImagFileName  = BrainVoyager.FindChild("btnGetImagFileName");
    btnConvertCartFiles2PolFiles = BrainVoyager.FindChild("btnConvertCartFiles2PolFiles");
    lblRealFileName     = BrainVoyager.FindChild("lblRealFileName"); 
    lblImagFileName     = BrainVoyager.FindChild("lblImagFileName");
    btnLoadText         = BrainVoyager.FindChild("btnLoadText");
    rbtnLoadText        = BrainVoyager.FindChild("rbtnLoadText");
    rbtnFilesGUI        = BrainVoyager.FindChild("rbtnFilesGUI");
    lblTxtFile          = BrainVoyager.FindChild("lblTxtFile");
    
    // action widgets
    proceedButton       = BrainVoyager.FindChild("btnProceed");
    startButton         = BrainVoyager.FindChild("btnStart");
    
    // evaluation widgets
    lblGetDistImg = BrainVoyager.FindChild("lblGetDistImg");
    lblGetUndistImg = BrainVoyager.FindChild("lblGetUndistImg");
    btnCreateDiffImg = BrainVoyager.FindChild("btnCreateDiffImg");
    
    // conversion widgets
    input1Label = BrainVoyager.FindChild("input1Label");
    input2Label = BrainVoyager.FindChild("input2Label"); 
    input3Label = BrainVoyager.FindChild("input3Label"); 
    dspbInput1 = BrainVoyager.FindChild("dspbInput1");
    dspbInput2 = BrainVoyager.FindChild("dspbInput2"); 
    dspbInput3 = BrainVoyager.FindChild("dspbInput3");  
    outputLabel = BrainVoyager.FindChild("outputLabel");    
    outputField = BrainVoyager.FindChild("outputField"); 
    btnConvertOtherCalc = BrainVoyager.FindChild("btnConvertOtherCalc");     
    btnConvertCart2Pol = BrainVoyager.FindChild("btnConvertCart2Pol");
    btnConvertPol2Cart = BrainVoyager.FindChild("btnConvertPol2Cart");
    leRealCoord = BrainVoyager.FindChild("leRealCoord");
    leImagCoord = BrainVoyager.FindChild("leImagCoord");
    leMagn = BrainVoyager.FindChild("leMagn");
    lePhase = BrainVoyager.FindChild("lePhase");
    fieldSIamount = BrainVoyager.FindChild("leSIinput");
    boxSIamountTime = BrainVoyager.FindChild("cmbTime");
    boxSIamountDistance = BrainVoyager.FindChild("cmbDistance");
    boxSIamountFrequency = BrainVoyager.FindChild("cmbFrequency");
    btnConvertSIunits = BrainVoyager.FindChild("btnConvertSIunits");
    secondsField = BrainVoyager.FindChild("leSeconds"); 
    metersField = BrainVoyager.FindChild("leMeters");
    hertzField = BrainVoyager.FindChild("leHertz");
  /* btnConvertPPMFile2Hz = BrainVoyager.FindChild("btnConvertPPMFile2Hz");
   btnGetPPMFileName = BrainVoyager.FindChild("btnGetPPMFileName");
  */
   rbtnDeg2Rad = BrainVoyager.FindChild("rbtnDeg2Rad");
   rbtnRad2Deg = BrainVoyager.FindChild("rbtnRad2Deg");
   rbtnRad2Hz = BrainVoyager.FindChild("rbtnRad2Hz");
   rbtnPPM2Hz = BrainVoyager.FindChild("rbtnPPM2Hz");
   rbtnES2PEBW = BrainVoyager.FindChild("rbtnES2PEBW");
   rbtnDFreq2Dx = BrainVoyager.FindChild("rbtnDFreq2Dx");

    // connect proceed to file uploading button with function
    proceedButton.clicked.connect(this, this.proceedToFileUpload);
    
    // connect file buttons to file dialog functions
    magnFileButton.clicked.connect(this, this.getMagnFile);
    phaseFileButton.clicked.connect(this, this.getPhaseFile);
    unwrappedFileButton.clicked.connect(this, this.getUnwrappedFiles);
    b0FileButton.clicked.connect(this, this.getB0Files);  
    pixshiftFileButton.clicked.connect(this, this.getPixelshiftFiles); 
    epiFileButton.clicked.connect(this, this.getEPIFiles); 
    vmrFileButton.clicked.connect(this, this.getVMRFile);
    btnGetDistImg.clicked.connect(this, this.getDistortedImageFile);
    btnGetUndistImg.clicked.connect(this, this.getUndistortedImageFile);
    btnCreateDiffImg.clicked.connect(this, this.createDifferenceImage);
    btnLoadText.clicked.connect(this, this.getFileNamesFile);
        
    // conversions: other calculations    
    rbtnDeg2Rad.toggled.connect(this, this.setDeg2RadLabel);    
    rbtnRad2Deg.toggled.connect(this, this.setRad2DegLabel);
    rbtnRad2Hz.toggled.connect(this, this.setRad2HzLabel);
    rbtnPPM2Hz.toggled.connect(this, this.setPPM2HzLabels);
    rbtnES2PEBW.toggled.connect(this, this.setES2PEBWLabels);
    rbtnDFreq2Dx.toggled.connect(this, this.setDFreq2DxLabels);
    btnConvertOtherCalc.clicked.connect(this, this.convertOtherEntities); 
        
    btnGetRealFileName.clicked.connect(this, this.getRealFileName);
    btnGetImagFileName.clicked.connect(this, this.getImaginaryFileName);
   // btnGetPPMFileName.clicked.connect(this, this.getPPMFileName);// 2. evaluate button

    
    // 3. conversion buttons
    btnConvertCart2Pol.clicked.connect(this, this.convertCartesianToPolar);
   btnConvertPol2Cart.clicked.connect(this, this.convertPolarToCartesian); 
    btnConvertSIunits.clicked.connect(this, this.convertSIunits);
   btnConvertCartFiles2PolFiles.clicked.connect(this, this.convertCartesianToPolarFiles);
 /*  btnConvertPPMFile2Hz.clicked.connect(this, this.convertPPMFile2Hz);
   */
    // connect 'Start processing' button to start processing function
    startButton.clicked.connect(this, this.startProcessing);
    
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/* constructor of complex number object */
function Complex(x, y, isPolar) {

    BrainVoyager.PrintToLog("Create complex number...");
    if (isPolar) {
        	this.real =	x * Math.cos(y);	
			this.imag = x * Math.sin(y);	
    } else {
        this.real = x;
        this.imag = y;
    }    
    BrainVoyager.PrintToLog(this.real + " + " + this.imag + "i");
    
    // declaration of member functions
    this.magnitude = magnitude;
    this.phase = phase;
}

function magnitude() {
    var magnitude = Math.sqrt(Math.pow(this.real,2) + Math.pow(this.imag,2));
    return magnitude; 
}

function phase() {
    var phase = Math.atan2(this.imag, this.real);
    return phase; 
}
/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

// This function adds a substring at the end of the name (before the extension): <old name>_<new string>.<old extension>
function extendFileName(filename, substr) {

	var index = filename.lastIndexOf(".");
	if (index != -1) {
		var basename = filename.substring(0, index);
		var extension = filename.substring(index, filename.length);
		var newname = basename + "_" + substr + extension;
	}
	return newname;
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
